@@ -39,6 +39,7 @@ namespace Beagle.Filters {
 	public class FilterOpenOffice : Beagle.Daemon.Filter {
 
 		Hashtable hotStyles;
+		bool odtFormat;
 		
 		public FilterOpenOffice () 
 		{
@@ -46,6 +47,7 @@ namespace Beagle.Filters {
 			AddSupportedMimeType ("application/vnd.sun.xml.impress");
 			AddSupportedMimeType ("application/vnd.sun.xml.calc");
 			SnippetMode = true;
+			odtFormat = false;
 		}
 		
 		static String FindChildAttribute (XmlNode node,
@@ -79,7 +81,8 @@ namespace Beagle.Filters {
 				reader.Read ();
 				while (reader.Depth > original_depth) {
 					if (reader.NodeType == XmlNodeType.Element
-					    && reader.Name == "style:properties") {
+					    && (reader.Name == "style:properties" ||
+						reader.Name == "style:text-properties")) {  /* ODT changes */
 						weight = reader.GetAttribute ("fo:font-weight");
 						italic = reader.GetAttribute ("fo:font-style");
 						underline = reader.GetAttribute ("style:text-underline");
@@ -99,22 +102,31 @@ namespace Beagle.Filters {
 				hotStyles[style_name] = true;
 		}
 
+		static bool NodeIsLink (String nodeName)
+		{
+			return nodeName == "text:a";
+		}
+
 		static bool NodeIsHot (String nodeName)
 		{
 			return nodeName == "text:h";
 		}
 
+		// These container tags allows multiple-lines of texts and 
+		// all of them should be marked *HOT* and hence called Container ;-)
 		static bool NodeIsHotContainer (String nodeName)
 		{
 			return nodeName == "office:annotation" ||
 				nodeName == "text:footnote" ||
-				nodeName == "text:endnote";
+				nodeName == "text:endnote" ||
+				nodeName == "text:note"; // "ODT format"
 		}
 
 		static bool NodeIsFreezing (String nodeName)
 		{
 			return nodeName == "text:footnote-citation"
-				|| nodeName == "text:endnote-citation";
+				|| nodeName == "text:endnote-citation"
+				|| nodeName == "text:note-citation";  // "ODT format"
 
 		}
 
@@ -122,7 +134,8 @@ namespace Beagle.Filters {
 		{
 			return nodeName == "text:footnote"
 				|| nodeName == "text:endnote"
-				|| nodeName == "office:annotation";
+				|| nodeName == "office:annotation"
+				|| nodeName == "text:note";       // "ODT format"
 		}
 
 		static bool NodeBreaksTextAfter (String nodeName)
@@ -138,7 +151,8 @@ namespace Beagle.Filters {
 				|| nodeName == "text:h"
 				|| nodeName == "text:footnote"
 				|| nodeName == "text:endnote"
-				|| nodeName == "office:annotation";
+				|| nodeName == "office:annotation"
+				|| nodeName == "text:note";       // "ODT format"
 		}
 
 		private Stack hot_nodes = new Stack ();
@@ -148,7 +162,6 @@ namespace Beagle.Filters {
 
 		void AddTextForIndexing (string paramStr)
 		{
-			bool is_text_hot =  (bool) hot_nodes.Peek ();
 			int sindex = 0;
 			string strTemp;
 			bool wasHot = false;
@@ -191,7 +204,7 @@ namespace Beagle.Filters {
 			// Enable *HOT* just before appending the text
 			// because, there can be some *Partial Texts* without
 			// *HOT* styles that needs to be appended.
-			if ((bool)hot_nodes.Peek() == true) {
+			if (hot_nodes.Count > 0 && (bool) hot_nodes.Peek() == true) {
 				if (!IsHot)
 					HotUp ();
 				bPartHotStyle = true;
@@ -258,6 +271,11 @@ namespace Beagle.Filters {
 						isHot = true;
 					} else if (NodeIsHotContainer (reader.Name)) {
 						hot_container_nodes.Push (reader.Name);
+					} else if (NodeIsLink (reader.Name)) {
+						string attr = reader.GetAttribute ("xlink:href");
+						AppendText (attr);
+						AppendWhiteSpace ();
+						continue;
 					} else {
 						bool has_attr = reader.MoveToFirstAttribute ();
 						while (has_attr) {
@@ -272,7 +290,11 @@ namespace Beagle.Filters {
 					} 
 					
 					hot_nodes.Push (isHot);
-					if (isHot || hot_container_nodes.Count > 0)
+
+					// Call *HotUp* iff
+					//   i) Its already in *HOT* mode and
+					//  ii) there is a hot style/hot container tag
+					if (!IsHot && (isHot || hot_container_nodes.Count > 0))
 						HotUp ();
 				
 					if (NodeIsFreezing (reader.Name)) {
@@ -301,7 +323,11 @@ namespace Beagle.Filters {
 					else if (NodeBreaksTextAfter (reader.Name))
 						AppendWhiteSpace ();
 
-					bool is_hot = (bool) hot_nodes.Pop ();
+					bool is_hot = false;
+					if (hot_nodes.Count > 0)
+						is_hot = (bool) hot_nodes.Pop ();
+					else
+						Logger.Log.Debug ("FilterOpenOffice: hot_nodes underflow in {0}", reader.Name);
 					
 					if (hot_container_nodes.Count > 0) {
 						string hot_container_tag = (string) hot_container_nodes.Peek ();
@@ -365,8 +391,6 @@ namespace Beagle.Filters {
 
 		private void ExtractMetadata (XmlReader reader)
 		{
-			string slideCount = null;
-
 			do {
 				reader.Read ();
 			} while (reader.Depth < 2);
@@ -423,7 +447,6 @@ namespace Beagle.Filters {
 										  reader.Value));
 					}
 					break;
-					
 				}
 				
 				reader.Read ();
@@ -437,6 +460,8 @@ namespace Beagle.Filters {
 			hotStyles = new Hashtable ();
 			try {
 				zip = new ZipFile (info.FullName);
+				if (MimeType == "application/vnd.oasis.opendocument.text")
+					odtFormat = true;
 			} catch (Exception e) {
 				Logger.Log.Error ("Caught exception inside FilterOpenOffice.DoOpen");
 				Logger.Log.Error (e); 
