@@ -395,6 +395,7 @@ namespace Beagle.Filters {
 			textType = TextType.Invalid;
 			file = null;
 			FileName = null;
+			SnippetMode = true;
 		}
 
 		private int ParseElement (Input stream)
@@ -456,7 +457,7 @@ namespace Beagle.Filters {
 						if (IsHot)
 							HotDown ();
 						else
-							AppendWhiteSpace ();
+							AppendStructuralBreak ();
 					}  else if (opcode == RecordType.TypeCode.TextHeaderAtom) {
 						data = stream.Read (4);
 						textType = (TextType) GetInt32 (data, 0);
@@ -480,7 +481,7 @@ namespace Beagle.Filters {
 				sumMeta = Msole.MetadataReadReal (sumStream);
 			else
 				Logger.Log.Error ("SummaryInformationStream not found in {0}", FileName);
-			
+
 			DocMetaData docSumMeta = null;
 			if (docSumStream != null)
 				docSumMeta = Msole.MetadataReadReal (docSumStream);
@@ -552,14 +553,16 @@ namespace Beagle.Filters {
 			if (file == null)
 				return;
 
+			// FIXME: Should try to use Encoding instead of 
+			// string.IndexOf ()... Hacky stuff ;-)
 			childCount = file.NumChildren();
 			for (int i = 0; i < childCount && found != 2; i++) {
-				str = file.NameByIndex (i);
-				if (string.Compare (str, "SummaryInformation") == 0) {
+				str = file.NameByIndex (i);	
+				if (str.IndexOf ("SummaryInformation") > -1 && found < 1) {
 					sumStream = file.ChildByIndex (i);
 					found = 1;
 				}
-				else if (string.Compare (str, "DocumentSummaryInformation") == 0) {
+				else if (str.IndexOf ("DocumentSummaryInformation") > -1) {
 					docSumStream = file.ChildByIndex (i);
 					found = 2;
 				}
@@ -588,8 +591,9 @@ namespace Beagle.Filters {
 				// "Document" container.
 				while (!stream.Eof)
 					ParseElement (stream);
-			} else 
+			} else {
 				Logger.Log.Error ("Ole stream not found in {0}.  Content extraction skipped.", FileName);
+			}
 			Finished();
 		}
 
@@ -602,8 +606,24 @@ namespace Beagle.Filters {
 			if (input != null) {
 				input = input.Uncompress();
 				file = new InfileMSOle (input);
-			} else 
+			} else {
 				Logger.Log.Error ("Unable to open {0}", info.FullName);
+				Finished ();
+			}
+
+			// PPT 95/97-2000 format contains a "PP97_DUALSTORAGE", which is required
+			// to index PPT 97-2000 files.
+			// We don't support PPT 95 files, however, we happily accept patches ;-)
+
+			Input dualStorTemp = null;
+			if ((dualStorTemp = file.ChildByName ("PP97_DUALSTORAGE")) != null) {
+				// "PP97_DUALSTORAGE" is a storage containing some streams
+				if (dualStorTemp.Handle != IntPtr.Zero)
+					file = (Gsf.Infile) GLib.Object.GetObject (dualStorTemp.Handle);
+			} else if ((dualStorTemp = file.ChildByName ("Header")) != null) {
+				Logger.Log.Error ("{0} is a PPT 95 file.  Beagle does not support PPT 95 files. Skipping...", FileName);
+				Finished ();
+			}
 		}
 
 		// FIXME: These are utility functions and can be useful 
