@@ -33,6 +33,28 @@ namespace Beagle.Util {
 
 	public class Scheduler {
 
+		static private bool no_delays = false;
+		static private bool immediate_priority_only = false;
+
+		static Scheduler ()
+		{
+			if (Environment.GetEnvironmentVariable ("EXERCISE_THE_DOG") != null) {
+				Logger.Log.Warn ("***");
+				Logger.Log.Warn ("*** The EXERCISE_THE_DOG environment variable is deprecated.");
+				Logger.Log.Warn ("*** Please use BEAGLE_EXERCISE_THE_DOG in the future.");
+				Logger.Log.Warn ("*** We will now proceed to exercise the dog.  Thank you.");
+				Logger.Log.Warn ("***");
+				no_delays = true;
+			} else if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") != null) {
+				no_delays = true;
+			}
+	
+			if (Environment.GetEnvironmentVariable ("BEAGLE_IMMEDIATE_PRIORITY_ONLY") != null)
+				immediate_priority_only = true;
+		}
+
+		//////////////////////////////////////////////////////////////////////////////
+
 		static private Scheduler global = new Scheduler ();
 
 		static public Scheduler Global {
@@ -164,7 +186,12 @@ namespace Beagle.Util {
 				if (! cancelled) {
 					TouchAllTaskGroups ();
 					try {
+						Logger.Log.Debug ("Starting task {0}", Tag);
+						Stopwatch sw = new Stopwatch ();
+						sw.Start ();
 						DoTaskReal ();
+						sw.Stop ();
+						Logger.Log.Debug ("Finished task {0} in {1}", Tag, sw);
 					} catch (Exception ex) {
 						Logger.Log.Warn ("Caught exception in DoTaskReal");
 						Logger.Log.Warn ("        Tag: {0}", Tag);
@@ -389,6 +416,10 @@ namespace Beagle.Util {
 
 			lock (task_queue) {
 				if (task != null) {
+					
+					if (immediate_priority_only && task.Priority != Priority.Immediate)
+						return false;
+
 					old_task = task_by_tag [task.Tag] as Task;
 					if (old_task == task)
 						return true;
@@ -528,6 +559,7 @@ namespace Beagle.Util {
 		const double idle_ramp_up_time   = 5.271828 * 60; // probably should be longer
 		const double default_delayed_rate_factor =  9.03; // work about 1/10th of the time
 		const double default_idle_rate_factor    = 2.097; // work about 1/3rd of the time
+		const double default_maximum_delay       = 20;    // never wait for more than 20s
 
 		static DateTime first_time = DateTime.MinValue;
 		private double GetIdleTime ()
@@ -538,6 +570,9 @@ namespace Beagle.Util {
 		private double ComputeDelay (Priority priority_of_next_task,
 					     double   duration_of_previous_task)
 		{
+			if (no_delays)
+				return 0;
+
 			double rate_factor;
 
 			rate_factor = 2.0;
@@ -585,6 +620,9 @@ namespace Beagle.Util {
 			    && delay < 0.5)
 				delay = 0.5;
 
+			if (delay > default_maximum_delay)
+				delay = default_maximum_delay;
+
 			return delay;
 		}
 
