@@ -110,19 +110,19 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
-			Logger.Log.Debug ("Command Line: {0}",
-					  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
-
 
 		}
 
-		private static void OnServiceOwnerChanged (string serviceName,
-							   string oldOwner,
-							   string newOwner)
+		private static void OnNameOwnerChanged (string name,
+							string oldOwner,
+							string newOwner)
 		{
-			if (serviceName == "com.novell.Beagle" && newOwner == "") {
-				DBusisms.BusDriver.ServiceOwnerChanged -= OnServiceOwnerChanged;
+			if (name == "com.novell.Beagle" && newOwner == "") {
+#if HAVE_OLD_DBUS
+				DBusisms.BusDriver.ServiceOwnerChanged -= OnNameOwnerChanged;
+#else
+				DBusisms.BusDriver.NameOwnerChanged -= OnNameOwnerChanged;
+#endif
 				Application.Quit ();
 			}
 		}
@@ -132,14 +132,18 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Info ("Attempting to replace another beagled.");
 			DBus.Service service = DBus.Service.Get (DBusisms.Connection, "com.novell.Beagle");
-			DBusisms.BusDriver.ServiceOwnerChanged += OnServiceOwnerChanged;
+#if HAVE_OLD_DBUS
+			DBusisms.BusDriver.ServiceOwnerChanged += OnNameOwnerChanged;
+#else
+			DBusisms.BusDriver.NameOwnerChanged += OnNameOwnerChanged;
+#endif
 			do {
 				Logger.Log.Info ("Building Remote Control Proxy");
 				RemoteControlProxy proxy = RemoteControl.GetProxy (service);
 				Logger.Log.Info ("Sending Shutdown");
 				proxy.Shutdown ();
 				Application.Run ();
-			} while (! DBusisms.InitService ());
+			} while (! DBusisms.InitService (Beagle.DBusisms.Name));
 
 			return 0;
 		} 
@@ -160,19 +164,20 @@ namespace Beagle.Daemon {
 				Thread.Sleep (1000);
 			}
 		}
-		
+
 		public static int Main (string[] args)
 		{
 			// Process the command-line arguments
 
 			bool arg_replace = false;
 			bool arg_debug = false;
-			bool arg_debug_inotify = false;
 			bool arg_debug_memory = false;
-			bool arg_network = false;
 			bool arg_fg = false;
 			bool arg_disable_scheduler = false;
+#if ENABLE_NETWORK
+			bool arg_network = false;
 			int arg_port = 0;
+#endif
 
 			int i = 0;
 			while (i < args.Length) {
@@ -201,11 +206,6 @@ namespace Beagle.Daemon {
 					arg_debug = true;
 					break;
 
-				case "--debug-inotify":
-					arg_debug = true;
-					arg_debug_inotify = true;
-					break;
-
 				case "--debug-memory":
 					arg_debug = true;
 					arg_debug_memory = true;
@@ -223,14 +223,15 @@ namespace Beagle.Daemon {
 					++i; // we used next_arg
 					break;
 
-				case "--enable-network":
-					arg_network = true;
-					break;
-
 				case "--disable-scheduler":
 					arg_disable_scheduler = true;
 					break;
 
+#if ENABLE_NETWORK
+				case "--enable-network":
+					arg_network = true;
+					break;
+
 				case "--port":
 					try {
 						arg_port = Convert.ToInt32(next_arg);
@@ -239,6 +240,7 @@ namespace Beagle.Daemon {
 					}
 					++i; // we used next_arg
 					break;
+#endif
 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
@@ -254,24 +256,23 @@ namespace Beagle.Daemon {
 			if (arg_debug)
 				Logger.DefaultLevel = LogLevel.Debug;
 
-			if (arg_debug_inotify)
-				Inotify.Log.Level = LogLevel.Debug;
-			else {
-				// FIXME: A hard-wired constant for Inotify.Log doesn't really
-				// belong here.
-				// This overrides the default, so this logger is "decoupled".
-				Inotify.Log.Level = LogLevel.Warn;
-			}
+			Logger.LogToFile (PathFinder.LogDir, "Beagle", arg_fg);
+
+			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
+			
+			// FIXME: This try/catch is to work around a bug in mono 1.1.3
+			try {
+				Logger.Log.Debug ("Command Line: {0}",
+						  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
+			} catch (Exception ex) { }
 
-			bool echo = false;
-			if (arg_fg) {
-				// If we are running in the foreground, we want to echo the log to stdout.
-				echo = true;
-			}
-			SetupLog (echo);
 
 			// Make sure that extended attributes can be set.  If not, bail out with a message.
-			if (! ExtendedAttribute.Test (PathFinder.RootDir)) {
+			// FIXME FIXME FIXME: This assumes that EAs work the same on your storage dir
+			// as they do on your home dir, which is obviously not a good assumption.
+			// We just need to pick a file under the home dir with the right permissions
+			// and test it.
+			if (! ExtendedAttribute.Test (PathFinder.StorageDir)) {
 				Logger.Log.Fatal ("Could not set extended attributes on a file in your home "
 						  + "directory.  See "
 						  + "http://www.beaglewiki.org/index.php/Enable%20Extended%20Attributes "
@@ -290,8 +291,8 @@ namespace Beagle.Daemon {
 				DBusisms.Init ();
 				Application.Init ();
 
-				Logger.Log.Debug ("Acquiring com.novell.Beagle D-BUS service");
-				if (!DBusisms.InitService ()) {
+				Logger.Log.Debug ("Acquiring {0} D-BUS service", Beagle.DBusisms.Name);
+				if (!DBusisms.InitService (Beagle.DBusisms.Name)) {
 					if (arg_replace) {
 						ReplaceExisting ();
 					} else {
@@ -306,13 +307,16 @@ namespace Beagle.Daemon {
 				Logger.Log.Fatal ("Couldn't connect to the session bus.  "
 						  + "See http://beaglewiki.org/index.php/Installing%20Beagle "
 						  + "for information on setting up a session bus.");
-				Logger.Log.Fatal (e);
+				Logger.Log.Fatal (e.Message);
 				return 1;
 			} catch (Exception e) {
 				Logger.Log.Fatal ("Could not initialize Beagle's bus connection.");
 				Logger.Log.Fatal (e);
 				return 1;
 			}
+
+			// Set up our helper process and the associated monitoring
+			IndexHelperFu.Start ();
 			
 			// We want to spend as little time as possible
 			// between InitService and actually being able 
@@ -351,6 +355,11 @@ namespace Beagle.Daemon {
 
 #endif
 
+			// Set up out-of-process indexing
+			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
+				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
+
+
 			// Start the query driver.
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
@@ -395,6 +404,9 @@ namespace Beagle.Daemon {
 			// Shut down the global scheduler
 			Scheduler.Global.Stop ();
 
+			// Stop our indexing helper process
+			IndexHelperFu.Stop ();
+
 #if false
 			Logger.Log.Debug ("Unregistering Factory objects");
 			factory.UnregisterAll ();
