@@ -57,7 +57,7 @@ namespace Beagle.Daemon.EvolutionDataServerQueryable {
 
 		public EvolutionDataServerQueryable () : base ("AddressbookIndex")
 		{
-			string dir = Path.Combine (PathFinder.RootDir, "AddressbookIndex");
+			string dir = Path.Combine (PathFinder.StorageDir, "AddressbookIndex");
 			photo_dir = Path.Combine (dir, "Photos");
 			System.IO.Directory.CreateDirectory (photo_dir);
 
