@@ -33,11 +33,13 @@ using System.Threading;
 using System.Xml;
 
 using Beagle.Util;
+using Beagle.Daemon;
+
 using Camel = Beagle.Util.Camel;
 using GConf;
 using Mono.Posix;
 
-namespace Beagle.Daemon {
+namespace Beagle.Daemon.EvolutionMailDriver {
 
 	public abstract class EvolutionMailIndexableGenerator : IIndexableGenerator {
 
@@ -112,6 +114,11 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		static public bool SupportedContentType (GMime.ContentType type)
+		{
+			return type.Type == "text" || type.Type == "Text" || type.Type == "TEXT";
+		}
+
 		protected class PartHandler {
 			private MultiReader reader = null;
 			private int depth = 0;
@@ -135,11 +142,22 @@ namespace Beagle.Daemon {
 					GMime.Multipart multipart = (GMime.Multipart) part;
 
 					multipart.ForeachPart (new GMime.PartFunc (this.OnEachPart));
-				} else {
+				} else if (SupportedContentType (part.ContentType)) {
 					MemoryStream stream = new MemoryStream (part.GetData ());
 					StreamReader reader = new StreamReader (stream);
+
+					// If this is the first part, we need to skip past the
+					// message headers --- we want to store that data in
+					// properties, not index it as text.
+					if (this.reader.Count == 0) {
+						string line;
+						do {
+							line = reader.ReadLine ();
+						} while (line != null && line != "");
+					}
+
 					this.reader.Add (reader);
-				}
+				} 
 
 				--depth;
 			}
@@ -159,7 +177,7 @@ namespace Beagle.Daemon {
 
 		protected static Stream ReadAppData (string name)
 		{
-			string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 			return new FileStream (path, System.IO.FileMode.Open, FileAccess.Read, FileShare.Read);
 		}
 
@@ -181,14 +199,14 @@ namespace Beagle.Daemon {
 
 		protected static Stream WriteAppData (string name)
 		{
-			string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write, FileShare.None);
 		}
 
 		protected static void WriteAppDataLine (string name, string line)
 		{
 			if (line == null) {
-				string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+				string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 
 				if (File.Exists (path))
 					File.Delete (path);
@@ -220,6 +238,11 @@ namespace Beagle.Daemon {
 			else
 				return false;
 		}
+
+		public override int GetHashCode ()
+		{
+			return this.CrawlFile.FullName.GetHashCode ();
+		}
 	}
 
 	public class EvolutionMailIndexableGeneratorMbox : EvolutionMailIndexableGenerator {
@@ -601,8 +624,15 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			if (this.summary == null)
-				this.summary = Camel.Summary.load (this.summary_info.FullName);
+			if (this.summary == null) {
+				try {
+					this.summary = Camel.Summary.load (this.summary_info.FullName);
+				} catch (Exception e) {
+					EvolutionMailQueryable.log.Warn ("Unable to index {0}: {1}", this.folder_name,
+									 e.Message);
+					return false;
+				}
+			}
 
 			if (this.summary_enumerator == null)
 				this.summary_enumerator = this.summary.GetEnumerator ();
@@ -610,10 +640,11 @@ namespace Beagle.Daemon {
 			if (this.summary_enumerator.MoveNext ())
 				return true;
 
-			// FIXME: This is kind of a hack, but it's the only way with the IndexableGenerator
-			// to handle our removals.
 			foreach (string uid in this.deleted_list) {
 				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
+
+				// FIXME: This is kind of a hack, but it's the only way
+				// with the IndexableGenerator to handle our removals.
 				Scheduler.Task task = this.Queryable.NewRemoveTask (uri);
 				task.Priority = Scheduler.Priority.Immediate;
 				this.Queryable.ThisScheduler.Add (task);
@@ -676,9 +707,14 @@ namespace Beagle.Daemon {
 			return indexable;
 		}
 
+		private Uri CamelMessageUri (Camel.MessageInfo message_info)
+		{
+			return EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, message_info.uid);			
+		}
+
 		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, TextReader msgReader)
 		{
-			System.Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, messageInfo.uid);
+			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
 			indexable.Timestamp = messageInfo.Date;
