@@ -41,17 +41,37 @@ namespace Beagle.Daemon {
 
 		public FileAttributes Read (string path)
 		{
-			return ifas.Read (path);
+			lock (this) {
+				return ifas.Read (path);
+			}
+		}
+
+		public FileAttributes ReadOrCreate (string path)
+		{
+			lock (this) {
+				FileAttributes attr = ifas.Read (path);
+				if (attr == null) {
+					attr = new FileAttributes ();
+					attr.UniqueId = Guid.NewGuid ().ToString ();
+					attr.Path = path;
+					ifas.Write (attr);
+				}
+				return attr;
+			}
 		}
 
 		public bool Write (FileAttributes attr)
 		{
-			return ifas.Write (attr);
+			lock (this) {
+				return ifas.Write (attr);
+			}
 		}
 
 		public void Drop (string path)
 		{
-			ifas.Drop (path);
+			lock (this) {
+				ifas.Drop (path);
+			}
 		}
 
 		//////////////////////////////////////////////////////////
@@ -61,8 +81,9 @@ namespace Beagle.Daemon {
 			FileAttributes attr;
 
 			attr = Read (path);
-			
-			// FIXME: This check is incomplete
+
+			// FIXME: This check is incomplete, we should also check
+			// filter names, filter versions, etc.
 			return attr != null
 				&& attr.Path == path
 				&& FileSystem.GetLastWriteTime (path) <= attr.LastWriteTime;
@@ -70,10 +91,7 @@ namespace Beagle.Daemon {
 
 		public void AttachTimestamp (string path, DateTime mtime)
 		{
-			FileAttributes attr = new FileAttributes ();
-
-			// FIXME?: we just assume that there won't be collisions.
-			attr.UniqueId = Guid.NewGuid ().ToString ();
+			FileAttributes attr = ReadOrCreate (path);
 
 			attr.Path = path;
 			attr.LastWriteTime = mtime;
