@@ -24,8 +24,6 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-#define DBUS_IS_BROKEN_BROKEN_BROKEN
-
 using System;
 using System.Collections;
 using System.IO;
@@ -38,6 +36,9 @@ namespace Beagle.IndexHelper {
 
 	public class RemoteIndexerImpl : Beagle.Daemon.RemoteIndexerProxy {
 
+		static int close_count = 0;
+		static public int CloseCount { get { return close_count; } }
+
 		string name;
 		IIndexer indexer;
 		NextFlush next_flush;
@@ -177,11 +178,15 @@ namespace Beagle.IndexHelper {
 		override public bool Open ()
 		{
 			lock (this) {
-				Logger.Log.Debug ("Open on {0}", name);
+
+				// We allow the item to be opened multiple times,
+				// but close should only be called once.
 				if (is_open)
-					return true; // FIXME: Is this right?  Does it matter?
+					return true; 
+
 				is_open = true;
-				safe_to_close = false;
+				safe_to_close = true;
+
 				return Shutdown.WorkerStart (this, name);
 			}
 		}
@@ -193,6 +198,7 @@ namespace Beagle.IndexHelper {
 				Shutdown.WorkerFinished (this);
 				is_open = false;
 				queued_close = false;
+				++close_count;
 			}
 		}
 
@@ -242,7 +248,7 @@ namespace Beagle.IndexHelper {
 		override public void Remove (string uri_as_str)
 		{
 			if (indexer != null) {
-				Uri uri = new Uri (uri_as_str, true);
+				Uri uri = UriFu.UriStringToUri (uri_as_str);
 				next_flush.ToBeRemoved.Add (uri);
 			}
 		}
@@ -250,6 +256,7 @@ namespace Beagle.IndexHelper {
 		override public void Flush ()
 		{
 			if (indexer != null) {
+				safe_to_close = false;
 				NextFlush this_flush = next_flush;
 				next_flush = NewNextFlush ();
 				indexer.Flush ();
@@ -259,6 +266,11 @@ namespace Beagle.IndexHelper {
 			}
 		}
 
+		override public int GetItemCount ()
+		{
+			return indexer != null ? indexer.GetItemCount () : -1;
+		}
+
 	}
 
 }
