@@ -26,6 +26,7 @@
 
 
 using System;
+using System.IO;
 using System.Collections;
 using System.Reflection;
 using System.Text;
@@ -36,6 +37,8 @@ namespace Beagle.Daemon {
 	
 	public class QueryDriver {
 
+		// Enable or Disable specific Queryables by name
+
 		static ArrayList allowed_queryables = new ArrayList ();
 		static ArrayList denied_queryables = new ArrayList ();
 		
@@ -51,25 +54,33 @@ namespace Beagle.Daemon {
 
 		static private bool UseQueryable (string name)
 		{
+			if (allowed_queryables.Count == 0
+			    && denied_queryables.Count == 0)
+				return true;
+
 			name = name.ToLower ();
 
 			if (allowed_queryables.Count > 0) {
-				foreach (string allowed in allowed_queryables) {
-					if (name == allowed) 
+				foreach (string allowed in allowed_queryables)
+					if (name == allowed)
 						return true;
-				}
 				return false;
 			}
 
-			foreach (string denied in denied_queryables) {
+			foreach (string denied in denied_queryables)
 				if (name == denied)
 					return false;
-			}
 			return true;
 
 		}
 
+		//////////////////////////////////////////////////////////////////////////////////////
+
+		// Use introspection to find all classes that implement IQueryable, the construct
+		// associated Queryables objects.
+
 		static ArrayList queryables = new ArrayList ();
+		static Hashtable iqueryable_to_queryable = new Hashtable ();
 
 		static bool ThisApiSoVeryIsBroken (Type m, object criteria)
 		{
@@ -101,6 +112,11 @@ namespace Beagle.Daemon {
 						if (! UseQueryable (flavor.Name))
 							continue;
 
+						if (flavor.RequireInotify && ! Inotify.Enabled) {
+							Logger.Log.Debug ("Can't start backend '{0}' without inotify",  flavor.Name);
+							continue;
+						}
+
 						IQueryable iq = null;
 						try {
 							iq = Activator.CreateInstance (type) as IQueryable;
@@ -112,6 +128,7 @@ namespace Beagle.Daemon {
 						if (iq != null) {
 							Queryable q = new Queryable (flavor, iq);
 							queryables.Add (q);
+							iqueryable_to_queryable [iq] = q;
 							++count;
 							break;
 						}
@@ -121,6 +138,21 @@ namespace Beagle.Daemon {
 			Logger.Log.Debug ("Found {0} types in {1}", count, assembly.FullName);
 		}
 
+		static public void Start ()
+		{
+			ScanAssembly (Assembly.GetExecutingAssembly ());
+			
+			DirectoryInfo backends = new DirectoryInfo (PathFinder.BackendDir);
+
+			if (backends.Exists) {
+				foreach (FileInfo assembly in backends.GetFiles ("*.dll"))
+					ScanAssembly (Assembly.LoadFile (assembly.ToString ()));
+			}
+				
+			foreach (Queryable q in queryables)
+				q.Start ();
+		}
+
 		////////////////////////////////////////////////////////
 
 		public delegate void ChangedHandler (Queryable            queryable,
@@ -128,71 +160,77 @@ namespace Beagle.Daemon {
 
 		static public event ChangedHandler ChangedEvent;
 
-		static object worker_lock = new object ();
-
-
-		// FIXME: There should be a way to disconnect this OnQueryableChanged
-		// from the ChangedEvents.
-		static QueryDriver ()
+		// A method to fire the ChangedEvent event.
+		static public void QueryableChanged (IQueryable           iqueryable,
+						     IQueryableChangeData change_data)
 		{
-			// FIXME: We should look for Queryables in other assemblies.
-			ScanAssembly (Assembly.GetExecutingAssembly ());
-			foreach (Queryable q in queryables) {
-				q.ChangedEvent += OnQueryableChanged;
+			if (ChangedEvent != null) {
+				Queryable queryable = iqueryable_to_queryable [iqueryable] as Queryable;
+				ChangedEvent (queryable, change_data);
 			}
 		}
 
-		static public void Start ()
-		{
-			foreach (Queryable q in queryables)
-				q.Start ();
-		}
-
-		static public void DoQuery (QueryBody body, QueryResult result)
-		{
-			// The extra pair of calls to WorkerStart/WorkerFinished ensures that
-			// the QueryResult will fire the StartedEvent and FinishedEvent,
-			// even if no queryable accepts the query.
-			if (!result.WorkerStart (worker_lock)) {
-				return;
-			}
-			
-			try {
-				foreach (Queryable queryable in queryables) {
-					if (queryable.AcceptQuery (body))
-						queryable.DoQuery (body, result, null);
-				}
-			} finally {
-				result.WorkerFinished (worker_lock);
-			}
-		}
-
-		static private void OnQueryableChanged (Queryable            source,
-							IQueryableChangeData changeData)
-		{
-			if (ChangedEvent != null)
-				ChangedEvent (source, changeData);
-		}
-
 		////////////////////////////////////////////////////////
 
-		static public string GetHumanReadableStatus ()
-		{
-			StringBuilder builder = new StringBuilder ("\n");
+		private class QueryClosure : IQueryWorker {
 
-			foreach (Queryable q in queryables) {
-				if (builder.Length > 1)
-					builder.Append ("\n--------------------------------------\n\n");
-				builder.Append (q.Name);
-				builder.Append (":\n");
-				builder.Append (q.GetHumanReadableStatus ());
-				builder.Append ("\n");
-			}
-			builder.Append ("\n");
+			Queryable queryable;
+			QueryBody body;
+			QueryResult result;
+			IQueryableChangeData change_data;
 			
-			return builder.ToString ();
+			public QueryClosure (Queryable            queryable,
+					     QueryBody            body,
+					     QueryResult          result,
+					     IQueryableChangeData change_data)
+			{
+				this.queryable = queryable;
+				this.body = body;
+				this.result = result;
+				this.change_data = change_data;
+			}
+
+			public void DoWork ()
+			{
+				HitRegulator regulator = result.GetHitRegulator (queryable);
+				queryable.DoQuery (body, regulator, change_data);
+				regulator.Flush (result);
+			}
 		}
 
+		static public void DoOneQuery (Queryable            queryable,
+					       QueryBody            body,
+					       QueryResult          result,
+					       IQueryableChangeData change_data)
+		{
+			if (queryable.AcceptQuery (body)) {
+				QueryClosure qc = new QueryClosure (queryable, body, result, change_data);
+				result.AttachWorker (qc);
+			}
+		}
+
+		static public void DoQuery (QueryBody body, QueryResult result)
+		{
+			// The extra pair of calls to WorkerStart/WorkerFinished ensures:
+			// (1) that the QueryResult will fire the StartedEvent
+			// and FinishedEvent, even if no queryable accepts the
+			// query.
+			// (2) that the FinishedEvent will only get called when all of the
+			// backends have had time to finish.
+
+			object dummy_worker = new object ();
+
+			if (! result.WorkerStart (dummy_worker))
+				return;
+			
+			foreach (Queryable queryable in queryables)
+				DoOneQuery (queryable, body, result, null);
+			
+			result.WorkerFinished (dummy_worker);
+		}
+
+		////////////////////////////////////////////////////////
+
 		static public string GetIndexInformation ()
 		{
 			StringBuilder builder = new StringBuilder ("\n");
