@@ -24,8 +24,6 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-#define DBUS_IS_BROKEN_BROKEN_BROKEN
-
 using System;
 using System.Threading;
 using DBus;
@@ -180,7 +178,6 @@ namespace Beagle.Daemon {
 			UpToTheMainLoop up = UpToTheMainLoop.NewAdd (Proxy, indexable);
 			up.Run ();
 #else
-			Logger.Log.Debug ("+++ {0}", indexable.ToString ()); 
 			Proxy.Add (indexable.ToString ());
 #endif
 			++add_remove_count;
@@ -228,6 +225,11 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		public int GetItemCount ()
+		{
+			return Proxy.GetItemCount ();
+		}
+
 		private void OnProxyChanged (string list_of_added_uris_as_string,
 					     string list_of_removed_uris_as_string)
 		{
