@@ -44,7 +44,7 @@ namespace Beagle.Daemon {
 
 		static TextCache () 
 		{
-			text_cache_dir = Path.Combine (PathFinder.RootDir, "TextCache");
+			text_cache_dir = Path.Combine (PathFinder.StorageDir, "TextCache");
 			if (! Directory.Exists (text_cache_dir))
 				Directory.CreateDirectory (text_cache_dir);
 
@@ -71,7 +71,7 @@ namespace Beagle.Daemon {
 			if (create_new_db) {
 				DoNonQuery ("CREATE TABLE uri_index (            " +
 					    "  uri      STRING UNIQUE NOT NULL,  " +
-					    "  filename STRING UNIQUE NOT NULL   " +
+					    "  filename STRING NOT NULL          " +
 					    ")");
 			}
 		}
@@ -154,7 +154,7 @@ namespace Beagle.Daemon {
 				throw new Exception (msg);
 			}
 
-			lock (connection) 
+			lock (connection)
 				Insert (uri, SELF_CACHE_TAG);
 		}
 
@@ -163,7 +163,7 @@ namespace Beagle.Daemon {
 			string path = LookupPath (uri, true);
 
 			FileStream stream;
-			stream = new FileStream (path, FileMode.Create, FileAccess.Write, FileShare.Read);
+			stream = new FileStream (path, FileMode.Create, FileAccess.Write, FileShare.ReadWrite);
 
 			StreamWriter writer;
 			writer = new StreamWriter (stream);
@@ -178,7 +178,7 @@ namespace Beagle.Daemon {
 
 			FileStream stream;
 			try {
-				stream = new FileStream (path, FileMode.Open, FileAccess.Read, FileShare.Read);
+				stream = new FileStream (path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
 			} catch (FileNotFoundException ex) {
 				return null;
 			}
