@@ -27,99 +27,136 @@
 
 using System;
 using System.IO;
+using System.Threading;
 
 using Beagle.Daemon;
 using Beagle.Util;
 
 namespace Beagle.Daemon.TomboyQueryable {
 
-	[QueryableFlavor (Name="Tomboy", Domain=QueryDomain.Local)]
+	[QueryableFlavor (Name="Tomboy", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class TomboyQueryable : LuceneQueryable {
 
 		private static Logger log = Logger.Get ("TomboyQueryable");
 
-		string notesDir;
-		string backupDir;
+		string tomboy_dir;
+		int tomboy_wd = -1;
+		FileSystemWatcher tomboy_fsw = null;
 
-		int wdNotes = -1;
-		int wdBackup = -1;
-
-		public TomboyQueryable () : base (Path.Combine (PathFinder.RootDir, "TomboyIndex"))
+		public TomboyQueryable () : base ("TomboyIndex")
 		{
-			notesDir = Path.Combine (Environment.GetEnvironmentVariable ("HOME"), ".tomboy");
-			backupDir = Path.Combine (notesDir, "Backup");
+			tomboy_dir = Path.Combine (PathFinder.HomeDir, ".tomboy");
 		}
 
 		public override void Start () 
 		{
-			base.Start ();
+                        base.Start ();
 
-			// FIXME: We should do something more reasonable if
-			// ~/.tomboy doesn't exist.
-			if (! (Directory.Exists (notesDir) && Directory.Exists (backupDir)))
+                        ExceptionHandlingThread.Start (new ThreadStart (StartWorker));
+		}
+
+		private void StartWorker ()
+		{
+			if (!Directory.Exists (tomboy_dir) ) {
+				GLib.Timeout.Add (60000, new GLib.TimeoutHandler (CheckForExistence));
 				return;
-			
-			Inotify.EventType mask;
-			mask = Inotify.EventType.MovedTo
-				| Inotify.EventType.MovedFrom
-				| Inotify.EventType.CreateFile
-				| Inotify.EventType.DeleteFile
-				| Inotify.EventType.Modify;
+			}
 
-			wdNotes = Inotify.Watch (notesDir, mask);
-			wdBackup = Inotify.Watch (backupDir, mask);
+			if (Inotify.Enabled) {			
+				Inotify.EventType mask;
+				mask = Inotify.EventType.CreateFile
+					| Inotify.EventType.DeleteFile
+					| Inotify.EventType.CloseWrite;
+				
+				tomboy_wd = Inotify.Watch (tomboy_dir, mask);
 
-			Inotify.Event += OnInotifyEvent;
+				Inotify.Event += OnInotifyEvent;
+			} else {
+				FileSystemWatcher fsw = new FileSystemWatcher ();
+				fsw.Path = tomboy_dir;
+				fsw.Filter = "*.note";
+
+				fsw.Changed += new FileSystemEventHandler (OnChanged);
+				fsw.Created += new FileSystemEventHandler (OnChanged);
+				fsw.Deleted += new FileSystemEventHandler (OnDeleted);
+
+				fsw.EnableRaisingEvents = true;
+			}
 
 			// Crawl all of our existing notes to make sure that
 			// everything is up-to-date.
 			log.Info ("Scanning Tomboy notes...");
+
 			Stopwatch stopwatch = new Stopwatch ();
 			int count = 0;
 			stopwatch.Start ();
-			DirectoryInfo dir = new DirectoryInfo (notesDir);
+			DirectoryInfo dir = new DirectoryInfo (tomboy_dir);
+
 			foreach (FileInfo file in dir.GetFiles ()) {
 				if (file.Extension == ".note") {
 					IndexNote (file, Scheduler.Priority.Delayed);
 					++count;
 				}
 			}
+
 			stopwatch.Stop ();
 			log.Info ("Scanned {0} notes in {1}", count, stopwatch);
 		}
 
+		private bool CheckForExistence ()
+		{
+			if (!Directory.Exists (tomboy_dir))
+				return true;
+			
+			this.Start ();
+
+			return false;
+		}
+
+		/////////////////////////////////////////////////
+
+		// Modified/Created/Deleted event using Inotify
+
 		private void OnInotifyEvent (int wd,
 					     string path,
 					     string subitem,
 					     Inotify.EventType type,
 					     uint cookie)
 		{
-			if (wd != wdNotes && wd != wdBackup)
+			if (wd != tomboy_wd)
 				return;
 
-			// Ignore operations on the directories themselves
 			if (subitem == "")
 				return;
 
-			//Console.WriteLine ("*** {0} {1} {2}", path, subitem, type);
-
-			// Ignore backup files, tmp files, etc.
 			if (Path.GetExtension (subitem) != ".note")
 				return;
 			
-			if (wd == wdNotes && type == Inotify.EventType.MovedTo) {
+			if (type == Inotify.EventType.CloseWrite || type == Inotify.EventType.CreateFile) {
 				IndexNote (new FileInfo (Path.Combine (path, subitem)), Scheduler.Priority.Immediate);
-				//Console.WriteLine ("Indexed {0}", Path.Combine (path, subitem));
 			}
 
-			if (wd == wdBackup && type == Inotify.EventType.MovedTo) {
-				string oldPath = Path.Combine (notesDir, subitem);
-				RemoveNote (oldPath);
-				//Console.WriteLine ("Removing {0}", oldPath);
+			if (type == Inotify.EventType.DeleteFile) {
+				RemoveNote (Path.Combine (path, subitem));
 			}
-
 		}
 
+		// Modified/Created event using FSW
+
+		private void OnChanged (object o, FileSystemEventArgs args)
+		{
+			IndexNote (new FileInfo (args.FullPath), Scheduler.Priority.Immediate);
+		}
+
+		// Deleted event using FSW
+
+		private void OnDeleted (object o, FileSystemEventArgs args)
+		{
+			RemoveNote (args.FullPath);
+		}
+
+		/////////////////////////////////////////////////
+
 		private static Indexable NoteToIndexable (FileInfo file, Note note)
 		{
 			Indexable indexable = new Indexable (note.Uri);
@@ -144,7 +181,7 @@ namespace Beagle.Daemon.TomboyQueryable {
 
 		private void IndexNote (FileInfo file, Scheduler.Priority priority)
 		{
-			if (Driver.IsUpToDate (file.FullName))
+			if (this.FileAttributesStore.IsUpToDate (file.FullName))
 				return;
 
 			// Try and parse a Note from the given path
@@ -160,7 +197,8 @@ namespace Beagle.Daemon.TomboyQueryable {
 			task.SubPriority = 0;
 			ThisScheduler.Add (task);
 
-			// Write a plain-text version of our note out into the text cache
+			// Write a plain-text version of our note out into the
+			// text cache
 			try {
 				TextWriter writer = TextCache.GetWriter (note.Uri);
 				writer.Write (note.text);
