@@ -27,7 +27,9 @@
 using System;
 using System.IO;
 using System.Reflection;
-using BU = Beagle.Util;
+using Beagle.Util;
+
+using Gnome;
 
 namespace Beagle {
 	
@@ -59,6 +61,9 @@ namespace Beagle {
 		{
 			Stream stream;
 
+			if (name == null || name.Length == 0)
+				return null;
+
 			stream = GetStreamInner (name);
 			if (stream == null)
 				stream = GetStreamInner (name + ".png");
@@ -109,7 +114,10 @@ namespace Beagle {
 		static public string GetHtmlSourceForStock (string stockid,
 							    int size)
 		{
-			string path = BU.Icon.LookupName (stockid, size);
+			int base_size;
+			IconTheme icon_theme = new IconTheme ();
+			string path = icon_theme.LookupIcon (stockid, size, IconData.Zero, out base_size);
+
 			if (path != null && path != "") {
 				return "file://" + path;
 			}
@@ -135,7 +143,7 @@ namespace Beagle {
 			if (name.StartsWith ("file://")) {
 				return name;
 			} else if (name.StartsWith ("/")) {
-				return "file://" + name;
+				return StringFu.PathToQuotedFileUri (name);
 			} else {
 				Stream stream;
 
